@@ -1,17 +1,12 @@
-/*  $Revision: 1.2 $
+/*  $Id: convdate.c,v 1.3 1999/11/19 09:16:56 rra Exp $
 **
 **  Convert date strings and numbers to numbers and strings.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
 #include "clibrary.h"
 #include <ctype.h>
 #include <errno.h>
-#if	defined(DO_NEED_TIME)
 #include <time.h>
-#endif	/* defined(DO_NEED_TIME) */
-#include <sys/time.h>
+
 #include "libinn.h"
 #include "macros.h"
 
