@@ -1,4 +1,4 @@
-/*  $Revision: 1.14 $
+/*  $Revision: 1.2 $
 **
 **  Connect to the NNTP server and feed one article.
 */
@@ -151,7 +151,7 @@ main(ac, av)
     }
 
     /* Connect to the server. */
-    if (NNTPremoteopen(&FromServer, &ToServer, buff) < 0
+    if (NNTPremoteopen(NNTP_PORT, &FromServer, &ToServer, buff) < 0
      || FromServer == NULL
      || ToServer == NULL) {
 	if (buff[0])
