@@ -1,9 +1,10 @@
-/*  $Revision: 1.1 $
-/*  $Revision: 1.1 $
+/*  $Revision: 1.1.1.1 $
 **
 */
 #include <stdio.h>
 #include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include <sys/uio.h>
 
 
@@ -18,7 +19,7 @@ writev(fd, vp, vpcount)
 {
     int			count;
 
-    for (count = 0; --vpcount >= 0; count += vp->count, vp++)
+    for (count = 0; --vpcount >= 0; count += vp->iov_len, vp++)
 	if (xwrite(fd, vp->iov_base, vp->iov_len) < 0)
 	    return -1;
     return count;
