@@ -1,5 +1,5 @@
 #
-# $Id: filter_nnrpd.pl,v 1.3 1996/11/10 22:08:26 brister Exp $
+# $Id: filter_nnrpd.pl,v 1.1.1.1 1997/08/04 04:03:49 gpalmer Exp $
 #
 # Sample perl filtering code for nnrpd hook.
 #
@@ -17,7 +17,7 @@
 # Sample filter
 #
 sub filter_post {
-    my ($rval) = "" ;		# assume we'll accept.
+    my $rval = "" ;		# assume we'll accept.
 
 ### Uncomment this next block to reject articles that have 'make money'
 ### in their subject, or which have a "Re: " subject, but no References:
